@@ -84,6 +84,8 @@ if TYPE_CHECKING:
     from types import TracebackType
     from typing_extensions import Literal
     from meadowrun.meadowrun_pb2 import Job, ProcessState
+    from types_aiobotocore_s3 import S3Client
+    from types_aiobotocore_sqs import SQSClient
 
 # SEE ALSO ec2_alloc_stub.py
 
@@ -94,7 +96,7 @@ _U = TypeVar("_U")
 # replicate into each region.
 _AMIS = {
     "plain": {
-        "us-east-2": "ami-093bc850d42012c90",
+        "us-east-2": "ami-0c2ec58a802340753",
         "us-east-1": "ami-0a519867d2df1d0db",
         "us-west-1": "ami-036f320911e0afa56",
         "us-west-2": "ami-09a6c93dc39a05f37",
@@ -129,13 +131,13 @@ def ensure_ec2_alloc_table(region_name: str) -> None:
             TableName=_EC2_ALLOC_TABLE_NAME,
             AttributeDefinitions=[
                 {
-                    "AttributeName": _PUBLIC_ADDRESS,
+                    "AttributeName": _INSTANCE_ID,
                     "AttributeType": "S",
                 }
             ],
             KeySchema=[
                 {
-                    "AttributeName": _PUBLIC_ADDRESS,
+                    "AttributeName": _INSTANCE_ID,
                     "KeyType": "HASH",
                 }
             ],
@@ -299,19 +301,19 @@ class EC2InstanceRegistrar(InstanceRegistrar[_InstanceState]):
             for item in response["Items"]
         ]
 
-    async def get_registered_instance(self, public_address: str) -> _InstanceState:
+    async def get_registered_instance(self, name: str) -> _InstanceState:
         result = self._table.get_item(
-            Key={_PUBLIC_ADDRESS: public_address},
+            Key={_INSTANCE_ID: name},
             ProjectionExpression=",".join(
-                [_RUNNING_JOBS, _INSTANCE_ID, _PREVENT_FURTHER_ALLOCATION]
+                [_RUNNING_JOBS, _PUBLIC_ADDRESS, _PREVENT_FURTHER_ALLOCATION]
             ),
         )
         if "Item" not in result:
-            raise ValueError(f"ec2 instance {public_address} was not found")
+            raise ValueError(f"ec2 instance {name} was not found")
 
         return _InstanceState(
-            public_address,
-            result["Item"][_INSTANCE_ID],
+            result["Item"][_PUBLIC_ADDRESS],
+            name,
             # similar to above, also a hack. We know we won't need the
             # available_resources in the context that this function is called
             None,
@@ -367,7 +369,7 @@ class EC2InstanceRegistrar(InstanceRegistrar[_InstanceState]):
 
         success, result = ignore_boto3_error_code(
             lambda: self._table.update_item(
-                Key={_PUBLIC_ADDRESS: instance.public_address},
+                Key={_INSTANCE_ID: instance.name},
                 # subtract resources that we're allocating
                 UpdateExpression="SET " + ", ".join(set_expressions),
                 # Check to make sure the allocation is still valid
@@ -408,7 +410,7 @@ class EC2InstanceRegistrar(InstanceRegistrar[_InstanceState]):
 
         success, result = ignore_boto3_error_code(
             lambda: self._table.update_item(
-                Key={_PUBLIC_ADDRESS: instance.public_address},
+                Key={_INSTANCE_ID: instance.name},
                 UpdateExpression=(
                     f"SET {', '.join(set_expressions)} REMOVE {_RUNNING_JOBS}.#job_id"
                 ),
@@ -421,11 +423,9 @@ class EC2InstanceRegistrar(InstanceRegistrar[_InstanceState]):
         )
         return success
 
-    async def set_prevent_further_allocation(
-        self, public_address: str, value: bool
-    ) -> bool:
+    async def set_prevent_further_allocation(self, name: str, value: bool) -> bool:
         self._table.update_item(
-            Key={_PUBLIC_ADDRESS: public_address},
+            Key={_INSTANCE_ID: name},
             UpdateExpression=(
                 f"SET {_PREVENT_FURTHER_ALLOCATION}=:value, {_LAST_UPDATE_TIME}=:now"
             ),
@@ -482,7 +482,15 @@ class EC2InstanceRegistrar(InstanceRegistrar[_InstanceState]):
             else:
                 security_group_ids = alloc_cloud_instances.security_group_ids
         else:
-            security_group_ids = [get_ssh_security_group_id(region_name)]
+            if subnet_id is not None:
+                subnet = boto3.client("ec2", region_name=region_name).describe_subnets(
+                    SubnetIds=[subnet_id]
+                )["Subnets"][0]
+                vpc_id = subnet["VpcId"]
+            else:
+                vpc_id = None
+
+            security_group_ids = [get_ssh_security_group_id(region_name, vpc_id)]
 
         if alloc_cloud_instances.iam_role_instance_profile:
             iam_role_instance_profile = alloc_cloud_instances.iam_role_instance_profile
@@ -505,8 +513,17 @@ class EC2InstanceRegistrar(InstanceRegistrar[_InstanceState]):
             abort=abort,
         )
 
-    async def authorize_current_ip(self) -> None:
-        await authorize_current_ip_for_meadowrun_ssh(self.get_region_name())
+    async def authorize_current_ip(self, alloc_cloud_instances: AllocVM) -> None:
+        if not isinstance(alloc_cloud_instances, AllocEC2Instance):
+            # TODO do this in the type checker somehow
+            raise ValueError(
+                "Programming error: EC2InstanceRegistrar can only be used with "
+                "AllocEC2Instance"
+            )
+
+        await authorize_current_ip_for_meadowrun_ssh(
+            self.get_region_name(), alloc_cloud_instances.subnet_id
+        )
 
     async def open_ports(
         self,
@@ -709,30 +726,60 @@ class EC2GridJobInterface(GridJobCloudInterface):
         # any Job.job_ids
         self._job_id = str(uuid.uuid4())
 
+        self._sqs_client: Optional[SQSClient] = None
+        self._s3_client: Optional[S3Client] = None
+
+    async def __aenter__(self) -> EC2GridJobInterface:
+        session = aiobotocore.session.get_session()
+        self._sqs_client = await session.create_client(
+            "sqs", region_name=self._region_name
+        ).__aenter__()
+        self._s3_client = await session.create_client(
+            "s3", region_name=self._region_name
+        ).__aenter__()
+        return self
+
+    async def __aexit__(
+        self,
+        exc_typ: Type[BaseException],
+        exc_val: BaseException,
+        exc_tb: TracebackType,
+    ) -> None:
+        if self._sqs_client is not None:
+            await self._sqs_client.__aexit__(exc_typ, exc_val, exc_tb)
+        if self._s3_client is not None:
+            await self._s3_client.__aexit__(exc_typ, exc_val, exc_tb)
+
     def create_instance_registrar(self) -> InstanceRegistrar:
         return EC2InstanceRegistrar(self._region_name, "create")
 
     async def setup_and_add_tasks(self, tasks: Sequence[_T]) -> None:
+        if self._sqs_client is None or self._s3_client is None:
+            raise ValueError("EC2GridJobInterface must be created with `async with`")
+
         # create SQS queues and add tasks to the request queue
         print(f"The current run_map's id is {self._job_id}")
-        session = aiobotocore.session.get_session()
-        async with session.create_client(
-            "sqs", region_name=self._region_name
-        ) as sqs, session.create_client("s3", region_name=self._region_name) as s3c:
-            self._request_queue_url = asyncio.create_task(
-                _create_request_queue(self._job_id, sqs)
+        self._request_queue_url = asyncio.create_task(
+            _create_request_queue(self._job_id, self._sqs_client)
+        )
+        self._task_argument_ranges = asyncio.create_task(
+            _add_tasks(
+                self._job_id,
+                await self._request_queue_url,
+                self._s3_client,
+                self._sqs_client,
+                tasks,
             )
-            self._task_argument_ranges = asyncio.create_task(
-                _add_tasks(self._job_id, await self._request_queue_url, s3c, sqs, tasks)
-            )
-            await self._task_argument_ranges
+        )
+        await self._task_argument_ranges
 
-    async def ssh_host_from_address(self, address: str) -> SshHost:
+    async def ssh_host_from_address(self, address: str, instance_name: str) -> SshHost:
         return SshHost(
             address,
             SSH_USER,
             self._ssh_private_key,
             (self._cloud_provider, self._region_name),
+            instance_name,
         )
 
     async def shutdown_workers(self, num_workers: int) -> None:
@@ -740,8 +787,10 @@ class EC2GridJobInterface(GridJobCloudInterface):
             raise ValueError(
                 "setup_and_add_tasks must be called before send_worker_done_messages"
             )
+        if self._sqs_client is None:
+            raise ValueError("EC2GridJobInterface must be created with `async with`")
         await _add_worker_shutdown_messages(
-            await self._request_queue_url, num_workers, self._region_name
+            await self._request_queue_url, num_workers, self._sqs_client
         )
 
     async def get_worker_function(
@@ -762,10 +811,14 @@ class EC2GridJobInterface(GridJobCloudInterface):
 
     async def receive_task_results(
         self, *, stop_receiving: asyncio.Event, workers_done: asyncio.Event
-    ) -> AsyncIterable[Tuple[int, int, ProcessState]]:
+    ) -> AsyncIterable[List[Tuple[int, int, ProcessState]]]:
+        if self._s3_client is None:
+            raise ValueError("EC2GridJobInterface must be created with `async with`")
+
         return receive_results(
             self._job_id,
             self._region_name,
+            self._s3_client,
             stop_receiving=stop_receiving,
             all_workers_exited=workers_done,
         )
@@ -773,11 +826,13 @@ class EC2GridJobInterface(GridJobCloudInterface):
     async def retry_task(self, task_id: int, attempts_so_far: int) -> None:
         if self._request_queue_url is None or self._task_argument_ranges is None:
             raise ValueError("setup_and_add_tasks must be called before retry_task")
+        if self._sqs_client is None:
+            raise ValueError("EC2GridJobInterface must be created with `async with`")
 
         await retry_task(
             await self._request_queue_url,
             task_id,
             attempts_so_far + 1,
             (await self._task_argument_ranges)[task_id],
-            self._region_name,
+            self._sqs_client,
         )
