@@ -38,19 +38,21 @@ if TYPE_CHECKING:
     )
 from meadowrun.docker_controller import get_registry_domain
 from meadowrun.meadowrun_pb2 import (
-    Credentials,
     AwsSecretProto,
     AzureSecretProto,
-    KubernetesSecretProto,
-    ContainerAtTag,
-    ContainerAtDigest,
-    EnvironmentSpec,
-    EnvironmentType,
     CodeZipFile,
+    ContainerAtDigest,
+    ContainerAtTag,
+    Credentials,
+    EnvironmentSpec,
+    EnvironmentSpecInCode,
+    EnvironmentType,
     GitRepoBranch,
     GitRepoCommit,
-    EnvironmentSpecInCode,
+    KubernetesSecretProto,
+    ServerAvailableContainer,
     ServerAvailableFolder,
+    ServerAvailableInterpreter,
 )
 from meadowrun.pip_integration import (
     pip_freeze_without_local_current_interpreter,
@@ -73,7 +75,7 @@ class Secret(abc.ABC):
         pass
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class AwsSecret(Secret):
     """
     An AWS secret
@@ -100,7 +102,7 @@ class AwsSecret(Secret):
         )
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class AzureSecret(Secret):
     """
     An Azure secret
@@ -137,7 +139,7 @@ class AzureSecret(Secret):
         )
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class KubernetesSecret(Secret):
     """
     A Kubernetes secret
@@ -163,6 +165,20 @@ class KubernetesSecret(Secret):
         )
 
 
+@dataclasses.dataclass(frozen=True)
+class PreinstalledInterpreter:
+    """
+    Represents an interpreter that has been pre-installed on the remote machine. This is
+    useful if you're using a [custom AMI](../../how_to/custom_ami).
+
+    Attributes:
+        path_to_interpreter: The path to the python executable, e.g.
+            `/var/myenv/bin/python`
+    """
+
+    path_to_interpreter: str
+
+
 class ContainerInterpreterBase(abc.ABC):
     """An abstract base class for specifying a container as a python interpreter"""
 
@@ -181,7 +197,7 @@ class ContainerInterpreterBase(abc.ABC):
         pass
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class ContainerInterpreter(ContainerInterpreterBase):
     """
     Specifies a container image. The container image must be configured so that running
@@ -193,16 +209,26 @@ class ContainerInterpreter(ContainerInterpreterBase):
         username_password_secret: An AWS or Azure secret that has a username and
             password for connecting to the container registry (as specified or implied
             in image_name). Only needed if the image/container registry is private.
+        always_use_local: If this is True, only looks for the image on the EC2 instance
+            and does not try to download the image from a container registry. This will
+            only work if you've preloaded the image into the AMI via [Use a custom AMI
+            on AWS](../../how_to/custom_ami)
     """
 
     repository_name: str
     tag: str = "latest"
     username_password_secret: Optional[Secret] = None
+    always_use_local: bool = False
 
     def get_interpreter_spec(
         self,
     ) -> Union[InterpreterDeployment, VersionedInterpreterDeployment]:
-        return ContainerAtTag(repository=self.repository_name, tag=self.tag)
+        if self.always_use_local:
+            return ServerAvailableContainer(
+                image_name=f"{self.repository_name}:{self.tag}"
+            )
+        else:
+            return ContainerAtTag(repository=self.repository_name, tag=self.tag)
 
     def _get_repository_name(self) -> str:
         return self.repository_name
@@ -211,7 +237,7 @@ class ContainerInterpreter(ContainerInterpreterBase):
         return self.username_password_secret
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class ContainerAtDigestInterpreter(ContainerInterpreterBase):
     """
     Like [ContainerInterpreter][meadowrun.ContainerInterpreter] but specifies a digest
@@ -224,16 +250,28 @@ class ContainerAtDigestInterpreter(ContainerInterpreterBase):
         username_password_secret: An AWS or Azure secret that has a username and
             password for connecting to the container registry (as specified or implied
             in image_name). Only needed if the image/container registry is private.
+        always_use_local: If this is True, only looks for the image on the EC2 instance
+            and does not try to download the image from a container registry. This will
+            only work if you've preloaded the image into the AMI via [Use a custom AMI
+            on AWS](../../how_to/custom_ami)
     """
 
     repository_name: str
     digest: str
     username_password_secret: Optional[Secret] = None
+    always_use_local: bool = False
 
     def get_interpreter_spec(
         self,
     ) -> Union[InterpreterDeployment, VersionedInterpreterDeployment]:
-        return ContainerAtDigest(repository=self.repository_name, digest=self.digest)
+        if self.always_use_local:
+            return ServerAvailableContainer(
+                image_name=f"{self.repository_name}@{self.digest}"
+            )
+        else:
+            return ContainerAtDigest(
+                repository=self.repository_name, digest=self.digest
+            )
 
     def _get_repository_name(self) -> str:
         return self.repository_name
@@ -263,7 +301,7 @@ class InterpreterSpecFile(abc.ABC):
     pass
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class CondaEnvironmentYmlFile(InterpreterSpecFile):
     """
     Specifies an environment.yml file generated by conda env export.
@@ -280,7 +318,7 @@ class CondaEnvironmentYmlFile(InterpreterSpecFile):
     additional_software: Union[Sequence[str], str, None] = None
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class PipRequirementsFile(InterpreterSpecFile):
     """
     Specifies a requirements.txt file generated by pip freeze.
@@ -301,7 +339,7 @@ class PipRequirementsFile(InterpreterSpecFile):
     additional_software: Union[Sequence[str], str, None] = None
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class PoetryProjectPath(InterpreterSpecFile):
     """
     Specifies a poetry project
@@ -334,7 +372,7 @@ class LocalInterpreter(abc.ABC):
         pass
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class LocalCurrentInterpreter(LocalInterpreter):
     """
     Specifies the current python interpreter.
@@ -407,7 +445,7 @@ class LocalCurrentInterpreter(LocalInterpreter):
         )
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class LocalCondaInterpreter(LocalInterpreter):
     """
     Specifies a locally installed conda environment
@@ -437,7 +475,7 @@ class LocalCondaInterpreter(LocalInterpreter):
         )
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class LocalPipInterpreter(LocalInterpreter):
     """
     Specifies a locally available interpreter. It can be a "regular" install of a python
@@ -587,6 +625,10 @@ class Deployment:
                         Credentials.Type.USERNAME_PASSWORD,
                     )
                 )
+        elif isinstance(interpreter, PreinstalledInterpreter):
+            interpreter_spec = ServerAvailableInterpreter(
+                interpreter_path=interpreter.path_to_interpreter
+            )
         else:
             raise ValueError(f"Unexpected type of interpreter {type(interpreter)}")
 
@@ -718,6 +760,10 @@ class Deployment:
                         Credentials.Type.USERNAME_PASSWORD,
                     )
                 )
+        elif isinstance(interpreter, PreinstalledInterpreter):
+            interpreter_spec = ServerAvailableInterpreter(
+                interpreter_path=interpreter.path_to_interpreter
+            )
         else:
             raise ValueError(f"Unexpected type of interpreter: {type(interpreter)}")
 
