@@ -49,6 +49,7 @@ from meadowrun.shared import unpickle_exception
 
 if TYPE_CHECKING:
     from meadowrun.credentials import UsernamePassword
+    from types import TracebackType
 
 
 _T = TypeVar("_T")
@@ -321,6 +322,7 @@ class SshHost(Host):
         username: str,
         private_key: asyncssh.SSHKey,
         cloud_provider: Optional[Tuple[CloudProviderType, str]] = None,
+        instance_name: Optional[str] = None,
     ) -> None:
         super().__init__()
         self.address = address
@@ -332,6 +334,7 @@ class SshHost(Host):
         # the job via the right InstanceRegistrar when we're done. region name indicates
         # where the InstanceRegistrar that we used to allocate this job is.
         self.cloud_provider = cloud_provider
+        self.instance_name = instance_name
         self._connect_task: Optional[asyncio.Task[asyncssh.SSHClientConnection]] = None
 
     def _connection_future(self) -> asyncio.Task[asyncssh.SSHClientConnection]:
@@ -765,26 +768,35 @@ class AllocVM(Host, abc.ABC):
                 "AllocEC2Instance and AllocAzureVM"
             )
 
-        driver = GridJobDriver(
-            self._create_grid_job_cloud_interface(), num_concurrent_tasks
-        )
-
-        run_worker_loops = asyncio.create_task(
-            driver.run_worker_functions(
-                self,
-                function,
-                pickle_protocol,
-                job_fields,
-                resources_required_per_task,
-                wait_for_result,
+        async with self._create_grid_job_cloud_interface() as cloud_interface:
+            driver = GridJobDriver(cloud_interface, num_concurrent_tasks)
+            run_worker_loops = asyncio.create_task(
+                driver.run_worker_functions(
+                    self,
+                    function,
+                    pickle_protocol,
+                    job_fields,
+                    resources_required_per_task,
+                    wait_for_result,
+                )
             )
-        )
-        async for result in driver.add_tasks_and_get_results(
-            args, max_num_task_attempts
-        ):
-            yield result
+            num_tasks_done = 0
+            async for result in driver.add_tasks_and_get_results(
+                args, max_num_task_attempts
+            ):
+                yield result
+                num_tasks_done += 1
 
-        await run_worker_loops
+            await run_worker_loops
+
+        # this is for extra safety--the only case where we don't get all of our results
+        # back should be if run_worker_loops throws an exception because there were
+        # worker failures
+        if num_tasks_done < len(args):
+            raise ValueError(
+                "Gave up retrieving task results, most likely due to worker failures. "
+                f"Received {num_tasks_done}/{len(args)} task results."
+            )
 
     @abc.abstractmethod
     def _create_grid_job_cloud_interface(self) -> GridJobCloudInterface:
@@ -815,6 +827,17 @@ class GridJobCloudInterface(abc.ABC, Generic[_T, _U]):
     instances and start workers.
     """
 
+    async def __aenter__(self) -> GridJobCloudInterface:
+        return self
+
+    async def __aexit__(
+        self,
+        exc_typ: Type[BaseException],
+        exc_val: BaseException,
+        exc_tb: TracebackType,
+    ) -> None:
+        pass
+
     @abc.abstractmethod
     def create_instance_registrar(self) -> InstanceRegistrar:
         ...
@@ -828,7 +851,7 @@ class GridJobCloudInterface(abc.ABC, Generic[_T, _U]):
         ...
 
     @abc.abstractmethod
-    async def ssh_host_from_address(self, address: str) -> SshHost:
+    async def ssh_host_from_address(self, address: str, instance_name: str) -> SshHost:
         ...
 
     @abc.abstractmethod
@@ -849,7 +872,7 @@ class GridJobCloudInterface(abc.ABC, Generic[_T, _U]):
     @abc.abstractmethod
     async def receive_task_results(
         self, *, stop_receiving: asyncio.Event, workers_done: asyncio.Event
-    ) -> AsyncIterable[Tuple[int, int, ProcessState]]:
+    ) -> AsyncIterable[List[Tuple[int, int, ProcessState]]]:
         ...
 
     @abc.abstractmethod
@@ -905,68 +928,78 @@ class GridJobDriver:
         yet).
         """
 
-        if (
-            job_fields["ports"]
-            and alloc_cloud_instance.get_cloud_provider() == "AzureVM"
-        ):
-            raise NotImplementedError(
-                "Opening ports on Azure is not implemented, please comment on "
-                "https://github.com/meadowdata/meadowrun/issues/126"
-            )
-
-        # we create an asyncio.task for this because it requires waiting for
-        # _cloud_interface.setup_and_add_tasks to complete. We don't want to just run
-        # this sequentially, though, because we want to start launching instances before
-        # setup_and_add_tasks is complete.
-        async def _get_pickled_worker_function() -> bytes:
-            return cloudpickle.dumps(
-                await self._cloud_interface.get_worker_function(user_function),
-                protocol=pickle_protocol,
-            )
-
-        pickled_worker_function_task = asyncio.create_task(
-            _get_pickled_worker_function()
-        )
-
-        # "inner_" on the parameter names is just to avoid name collision with outer
-        # scope
-        async def launch_worker_function(
-            inner_ssh_host: SshHost,
-            inner_worker_job_id: str,
-            log_file_name: str,
-            inner_instance_registrar: InstanceRegistrar,
-        ) -> JobCompletion:
-            job = Job(
-                job_id=inner_worker_job_id,
-                py_function=PyFunctionJob(
-                    pickled_function=await pickled_worker_function_task,
-                    pickled_function_arguments=pickle.dumps(
-                        ([inner_ssh_host.address, log_file_name], {}),
-                        protocol=pickle_protocol,
-                    ),
-                ),
-                **job_fields,
-            )
-
-            async def deallocator() -> None:
-                await inner_instance_registrar.deallocate_job_from_instance(
-                    await inner_instance_registrar.get_registered_instance(
-                        inner_ssh_host.address
-                    ),
-                    inner_worker_job_id,
-                )
-
-            return await inner_ssh_host.run_cloud_job(job, wait_for_result, deallocator)
-
         address_to_ssh_host: Dict[str, SshHost] = {}
-        workers_launched = 0
-        worker_shutdown_messages_sent = 0
-        workers_exited_unexpectedly = 0
         worker_tasks: List[Tuple[str, asyncio.Task[JobCompletion]]] = []
+
         workers_needed_changed_wait_task = asyncio.create_task(
             self._workers_needed_changed.wait()
         )
+        pickled_worker_function_task: Optional[asyncio.Task[bytes]] = None
+        async_cancel_exception = False
+
         try:
+            if (
+                job_fields["ports"]
+                and alloc_cloud_instance.get_cloud_provider() == "AzureVM"
+            ):
+                raise NotImplementedError(
+                    "Opening ports on Azure is not implemented, please comment on "
+                    "https://github.com/meadowdata/meadowrun/issues/126"
+                )
+
+            # we create an asyncio.task for this because it requires waiting for
+            # _cloud_interface.setup_and_add_tasks to complete. We don't want to just
+            # run this sequentially, though, because we want to start launching
+            # instances before setup_and_add_tasks is complete.
+            async def _get_pickled_worker_function() -> bytes:
+                return cloudpickle.dumps(
+                    await self._cloud_interface.get_worker_function(user_function),
+                    protocol=pickle_protocol,
+                )
+
+            pickled_worker_function_task = asyncio.create_task(
+                _get_pickled_worker_function()
+            )
+
+            # "inner_" on the parameter names is just to avoid name collision with outer
+            # scope
+            async def launch_worker_function(
+                inner_ssh_host: SshHost,
+                inner_worker_job_id: str,
+                log_file_name: str,
+                inner_instance_registrar: InstanceRegistrar,
+            ) -> JobCompletion:
+                assert pickled_worker_function_task is not None  # just for mypy
+
+                job = Job(
+                    job_id=inner_worker_job_id,
+                    py_function=PyFunctionJob(
+                        pickled_function=await pickled_worker_function_task,
+                        pickled_function_arguments=pickle.dumps(
+                            ([inner_ssh_host.address, log_file_name], {}),
+                            protocol=pickle_protocol,
+                        ),
+                    ),
+                    **job_fields,
+                )
+
+                async def deallocator() -> None:
+                    if inner_ssh_host.instance_name is not None:
+                        await inner_instance_registrar.deallocate_job_from_instance(
+                            await inner_instance_registrar.get_registered_instance(
+                                inner_ssh_host.instance_name
+                            ),
+                            inner_worker_job_id,
+                        )
+
+                return await inner_ssh_host.run_cloud_job(
+                    job, wait_for_result, deallocator
+                )
+
+            workers_launched = 0
+            worker_shutdown_messages_sent = 0
+            workers_exited_unexpectedly = 0
+
             async with self._cloud_interface.create_instance_registrar() as instance_registrar:  # noqa: E501
                 while True:
                     # TODO we should subtract workers_exited_unexpectedly from
@@ -993,13 +1026,13 @@ class GridJobDriver:
                             self._abort_launching_new_workers,
                         ):
                             for (
-                                public_address,
+                                (public_address, instance_name),
                                 worker_job_ids,
                             ) in allocated_hosts.items():
                                 ssh_host = address_to_ssh_host.get(public_address)
                                 if ssh_host is None:
                                     ssh_host = await self._cloud_interface.ssh_host_from_address(  # noqa: E501
-                                        public_address
+                                        public_address, instance_name
                                     )
                                     address_to_ssh_host[public_address] = ssh_host
                                 for worker_job_id in worker_job_ids:
@@ -1024,8 +1057,14 @@ class GridJobDriver:
                                     workers_launched += 1
 
                     # shutdown workers if they're no longer needed
-                    if new_workers_to_launch < 0:
-                        workers_to_shutdown = -new_workers_to_launch
+                    # once we have the lost worker replacement logic this should just be
+                    # -workers_to_launch
+                    workers_to_shutdown = (
+                        workers_launched
+                        - worker_shutdown_messages_sent
+                        - workers_exited_unexpectedly
+                    ) - (self._workers_needed)
+                    if workers_to_shutdown > 0:
                         await self._cloud_interface.shutdown_workers(
                             workers_to_shutdown
                         )
@@ -1075,12 +1114,28 @@ class GridJobDriver:
                     ):
                         break
 
-                # wait for any remaining workers to finish
-                self._no_workers_available.set()
-                for worker_task in worker_tasks:
-                    worker_task[1].cancel()
+        except asyncio.CancelledError:
+            # if we're being cancelled, then most likely the worker_tasks are being
+            # cancelled as well (because someone pressed Ctrl+C), which means it's
+            # unhelpful to cancel them again, we want them to finish running their
+            # except/finally clauses
+            async_cancel_exception = True
+            raise
         finally:
             print("Shutting down workers")
+            # setting this is very critical--otherwise, add_tasks_and_get_results will
+            # hang forever, not knowing that it has no hope of workers working on any of
+            # its tasks
+            self._no_workers_available.set()
+
+            if not async_cancel_exception:
+                # cancel any outstanding workers. Even if there haven't been any
+                # exception and we've sent worker shutdown messages for all the workers,
+                # a worker might be in the middle of building/pulling an environment and
+                # we don't want to wait for that to complete
+                for worker_task in worker_tasks:
+                    worker_task[1].cancel()
+
             # we still want the tasks to complete their except/finally blocks, after
             # which they should reraise asyncio.CancelledError, which we can safely
             # ignore
@@ -1095,6 +1150,9 @@ class GridJobDriver:
                 ],
                 return_exceptions=True,
             )
+            workers_needed_changed_wait_task.cancel()
+            if pickled_worker_function_task is not None:
+                pickled_worker_function_task.cancel()
 
     async def add_tasks_and_get_results(
         self,
@@ -1120,57 +1178,59 @@ class GridJobDriver:
             f"Waiting for task results. Requested: {len(args)}, "
             f"Done: {num_tasks_done}"
         )
-        async for task_id, attempt, result in await self._cloud_interface.receive_task_results(  # noqa: E501
+        async for batch in await self._cloud_interface.receive_task_results(  # noqa: E501
             stop_receiving=stop_receiving, workers_done=self._no_workers_available
         ):
-            t0 = time.time()
-            if t0 - last_printed_update > _PRINT_RECEIVED_TASKS_SECONDS:
-                print(
-                    f"Waiting for task results. Requested: {len(args)}, "
-                    f"Done: {num_tasks_done}"
-                )
-                last_printed_update = t0
-            if result.state == ProcessState.ProcessStateEnum.SUCCEEDED:
-                num_tasks_done += 1
-                # TODO try/catch on pickle.loads?
-                yield TaskResult(
-                    task_id,
-                    is_success=True,
-                    result=pickle.loads(result.pickled_result),
-                    attempt=attempt,
-                    log_file_name=result.log_file_name,
-                )
-            else:
-                if result.state in _EXCEPTION_STATES:
-                    exception = unpickle_exception(result.pickled_result)
-                    task_result: TaskResult = TaskResult(
-                        task_id,
-                        is_success=False,
-                        exception=exception,
-                        attempt=attempt,
-                        log_file_name=result.log_file_name,
-                    )
-                else:
-                    task_result = TaskResult(
-                        task_id,
-                        is_success=False,
-                        attempt=attempt,
-                        log_file_name=result.log_file_name,
-                    )
-
-                if attempt < max_num_task_attempts:
-                    print(f"Task {task_id} failed at attempt {attempt}, retrying.")
-                    await self._cloud_interface.retry_task(task_id, attempt)
-                else:
-                    print(
-                        f"Task {task_id} failed at attempt {attempt}, "
-                        f"max attempts is {max_num_task_attempts}, not retrying."
-                    )
+            for task_id, attempt, result in batch:
+                if result.state == ProcessState.ProcessStateEnum.SUCCEEDED:
                     num_tasks_done += 1
-                    yield task_result
+                    # TODO try/catch on pickle.loads?
+                    yield TaskResult(
+                        task_id,
+                        is_success=True,
+                        result=pickle.loads(result.pickled_result),
+                        attempt=attempt,
+                        log_file_name=result.log_file_name,
+                    )
+                else:
+                    if result.state in _EXCEPTION_STATES:
+                        exception = unpickle_exception(result.pickled_result)
+                        task_result: TaskResult = TaskResult(
+                            task_id,
+                            is_success=False,
+                            exception=exception,
+                            attempt=attempt,
+                            log_file_name=result.log_file_name,
+                        )
+                    else:
+                        task_result = TaskResult(
+                            task_id,
+                            is_success=False,
+                            attempt=attempt,
+                            log_file_name=result.log_file_name,
+                        )
+
+                    if attempt < max_num_task_attempts:
+                        print(f"Task {task_id} failed at attempt {attempt}, retrying.")
+                        await self._cloud_interface.retry_task(task_id, attempt)
+                    else:
+                        print(
+                            f"Task {task_id} failed at attempt {attempt}, "
+                            f"max attempts is {max_num_task_attempts}, not retrying."
+                        )
+                        num_tasks_done += 1
+                        yield task_result
 
             if num_tasks_done >= len(args):
                 stop_receiving.set()
+            else:
+                t0 = time.time()
+                if t0 - last_printed_update > _PRINT_RECEIVED_TASKS_SECONDS:
+                    print(
+                        f"Waiting for task results. Requested: {len(args)}, "
+                        f"Done: {num_tasks_done}"
+                    )
+                    last_printed_update = t0
 
             # reduce the number of workers needed if we have more workers than
             # outstanding tasks
@@ -1185,6 +1245,10 @@ class GridJobDriver:
         self._abort_launching_new_workers.set()
 
         if num_tasks_done < len(args):
+            # It would make sense for this to raise an exception, but it's more helpful
+            # to see the actual worker failures, and run_worker_functions should always
+            # raise an exception in that case. The caller should still check though that
+            # we returned all of the task results we were expecting.
             print(
                 "Gave up retrieving task results, most likely due to worker failures. "
                 f"Received {num_tasks_done}/{len(args)} task results."
