@@ -75,6 +75,7 @@ from meadowrun.run_job_core import (
     MeadowrunException,
     ObjectStorage,
     TaskResult,
+    WaitOption,
 )
 from meadowrun.shared import pickle_exception
 
@@ -426,10 +427,8 @@ async def _non_container_job_continuation(
         # wait for the process to finish
         # TODO add an optional timeout
 
-        with open(log_file_name, "wb") as log_file:
-            async for line in process.stdout:  # type: ignore
-                log_file.write(line)
-                sys.stdout.buffer.write(line)
+        async for line in process.stdout:  # type: ignore
+            sys.stdout.buffer.write(line)
         returncode = await process.wait()
         return _completed_job_state(
             job_spec_type,
@@ -620,10 +619,8 @@ async def _container_job_continuation(
         # that in a hacky way here.
         # TODO figure out overall strategy for logging, maybe eventually implement our
         #  own plain text/whatever log driver for docker.
-        with open(log_file_name, "w", encoding="utf-8") as f:
-            async for line in container.log(stdout=True, stderr=True, follow=True):
-                print(line, end="")
-                f.write(line)
+        async for line in container.log(stdout=True, stderr=True, follow=True):
+            print(line, end="")
 
         wait_result = await container.wait()
         # as per https://docs.docker.com/engine/api/v1.41/#operation/ContainerWait we
@@ -753,7 +750,7 @@ def _set_up_working_folder(
     # child processes
     io_folder = os.path.join(working_folder, "io")
     # holds the logs for the functions/commands that this server runs
-    job_logs_folder = os.path.join(working_folder, "job_logs")
+    job_logs_folder = "/var/meadowrun/job_logs"
     # see CodeDeploymentManager
     git_repos_folder = os.path.join(working_folder, "git_repos")
     # see CodeDeploymentManager
@@ -950,14 +947,21 @@ async def run_local(
         misc_folder,
     ) = _set_up_working_folder(working_folder)
 
-    # unpickle credentials if necessary
-    (
-        code_deployment_credentials,
-        interpreter_deployment_credentials,
-        all_sidecar_container_credentials,
-    ) = await _get_credentials_for_job(job, cloud)
+    # the logging actually happens via stdout redirection in the run_job_local_main
+    # caller
+    log_file_name = os.path.join(
+        job_logs_folder,
+        f"{job.job_friendly_name}.{job.job_id}.log",
+    )
 
     try:
+        # unpickle credentials if necessary
+        (
+            code_deployment_credentials,
+            interpreter_deployment_credentials,
+            all_sidecar_container_credentials,
+        ) = await _get_credentials_for_job(job, cloud)
+
         # first, get the code paths
         code_paths, interpreter_spec_path, cwd_path = await get_code_paths(
             git_repos_folder, local_copies_folder, job, code_deployment_credentials
@@ -1043,10 +1047,6 @@ async def run_local(
         job_spec_transformed.environment_variables.update(
             **_string_pairs_to_dict(job.environment_variables)
         )
-        log_file_name = os.path.join(
-            job_logs_folder,
-            f"{job.job_friendly_name}.{job.job_id}.log",
-        )
 
         # next we need to launch the job depending on how we've specified the
         # interpreter
@@ -1160,11 +1160,14 @@ async def run_local(
     except asyncio.CancelledError:
         raise
     except Exception as e:
+        traceback.print_exc()
+
         # we failed to launch the process
         return (
             ProcessState(
                 state=ProcessStateEnum.RUN_REQUEST_FAILED,
                 pickled_result=pickle_exception(e, job.result_highest_pickle_protocol),
+                log_file_name=log_file_name,
             ),
             None,
         )
@@ -1176,11 +1179,11 @@ class LocalHost(Host):
         self,
         resources_required: Optional[ResourcesInternal],
         job: Job,
-        wait_for_result: bool,
+        wait_for_result: WaitOption,
     ) -> JobCompletion[Any]:
-        if not wait_for_result:
+        if wait_for_result != WaitOption.WAIT_SILENTLY:
             raise NotImplementedError(
-                "wait_for_result=False is not supported for LocalHost yet"
+                f"{wait_for_result} is not supported for LocalHost yet"
             )
 
         if resources_required is not None:
@@ -1222,7 +1225,8 @@ class LocalHost(Host):
         job_fields: Dict[str, Any],
         num_concurrent_tasks: int,
         pickle_protocol: int,
-        wait_for_result: bool,
+        wait_for_result: WaitOption,
+        max_num_tasks_attempts: int,
     ) -> Optional[Sequence[_U]]:
         raise NotImplementedError("run_map on LocalHost is not implemented")
 
@@ -1234,6 +1238,8 @@ class LocalHost(Host):
         job_fields: Dict[str, Any],
         num_concurrent_tasks: int,
         pickle_protocol: int,
+        wait_for_result: WaitOption,
+        max_num_tasks_attempts: int,
     ) -> AsyncIterable[TaskResult[_U]]:
         raise NotImplementedError(
             "run_map_as_completed is not implemented for LocalHost"
