@@ -4,7 +4,7 @@ import asyncio
 import pickle
 import shutil
 import traceback
-from typing import Optional, TypeVar
+from typing import Optional, Tuple, TypeVar
 
 from meadowrun.meadowrun_pb2 import ProcessState
 
@@ -21,6 +21,10 @@ def pickle_exception(e: Exception, pickle_protocol: int) -> bytes:
     )
 
 
+def unpickle_exception(bs: bytes) -> Tuple[str, str, str]:
+    return pickle.loads(bs)
+
+
 COMPLETED_PROCESS_STATES = {
     ProcessState.ProcessStateEnum.SUCCEEDED,
     ProcessState.ProcessStateEnum.RUN_REQUEST_FAILED,
